package lcsb.mapviewer.services.impl;

import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.annotation.services.PubmedSearchException;
import lcsb.mapviewer.model.Article;
import lcsb.mapviewer.persist.dao.ArticleDao;
import lcsb.mapviewer.persist.dao.ArticleProperty;
import lcsb.mapviewer.services.interfaces.IPublicationService;

@Transactional
@Service
public class PublicationService implements IPublicationService {

  private static Logger logger = LogManager.getLogger();

  @Autowired
  private ArticleDao articleDao;

  @Autowired
  private PubmedParser pubmedParser;

  @Override
  public Page<Article> getByFilter(final Pageable pageable, final Map<ArticleProperty, Object> filterOptions) {
    fetchMissingForFilter(filterOptions);
    return articleDao.getByFilter(pageable, filterOptions);
  }

  private void fetchMissingForFilter(final Map<ArticleProperty, Object> filterOptions) {
    List<String> missingIds = articleDao.getMissingByFilter(filterOptions);
    for (final String id : missingIds) {
      try {
        Article article = pubmedParser.getPubmedArticleById(id);
        if (article != null && id.equals(article.getPubmedId())) {
          articleDao.add(article);
        }
      } catch (final PubmedSearchException e) {
        logger.error("Problem with accessing article", e);
      }
    }
  }

  @Override
  public long getCountByFilter(final Map<ArticleProperty, Object> filterOptions) {
    fetchMissingForFilter(filterOptions);
    return articleDao.getCountByFilter(filterOptions);
  }

  @Override
  public void add(final Article article) {
    articleDao.add(article);
  }

}
