package lcsb.mapviewer.web.api.project.overlay;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.model.map.BioEntity;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.overlay.DataOverlayEntry;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.IDataOverlayService;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IProjectService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.web.api.NewApiResponseSerializer;

@RestController
@Validated
@RequestMapping(
    value = {
        "/minerva/new_api/projects/{projectId}/overlays"
    },
    produces = MediaType.APPLICATION_JSON_VALUE)
public class NewOverlayController extends BaseController {

  private IModelService modelService;
  private IDataOverlayService dataOverlayService;
  private NewApiResponseSerializer serializer;

  public NewOverlayController(final IUserService userService,
      final IProjectService projectService,
      final IModelService modelService,
      final IDataOverlayService dataOverlayService,
      final NewApiResponseSerializer serializer) {
    this.dataOverlayService = dataOverlayService;
    this.serializer = serializer;
    this.modelService = modelService;
  }

  @PreAuthorize("hasAuthority('IS_ADMIN')"
      + " or hasAuthority('IS_CURATOR') and hasAuthority('READ_PROJECT:' + #projectId)"
      + " or hasAuthority('READ_PROJECT:' + #projectId) and "
      + "   (@dataOverlayService.getOverlayCreator(#overlayId)?.login == authentication.name "
      + "    or @dataOverlayService.getDataOverlayById(#overlayId)?.public == true)")
  @GetMapping(value = "/{overlayId}/models/{modelId}/bioEntities/")
  public ResponseEntity<?> getOverlayElements(
      final @PathVariable(value = "projectId") String projectId,
      final @PathVariable(value = "modelId") String modelId,
      final @PathVariable(value = "overlayId") Integer overlayId,
      final @RequestParam(value = "includeIndirect", defaultValue = "false") boolean includeIndirect)
      throws QueryException, IOException, ObjectNotFoundException {
    if (overlayId == null) {
      throw new QueryException("Invalid overlayId");
    }
    dataOverlayService.getDataOverlayById(projectId, overlayId);

    List<ModelData> models = modelService.getModelsByMapId(projectId, modelId);

    List<Pair<? extends BioEntity, DataOverlayEntry>> result = new ArrayList<>();
    for (ModelData model : models) {
      result.addAll(dataOverlayService.getBioEntitiesForDataOverlay(projectId, model.getId(), overlayId, true, includeIndirect));
    }

    return serializer.prepareResponse(result);
  }

}