package lcsb.mapviewer.web.api.project.chemicals;

import static org.junit.Assume.assumeTrue;
import static org.springframework.restdocs.mockmvc.MockMvcRestDocumentation.document;
import static org.springframework.restdocs.mockmvc.RestDocumentationRequestBuilders.get;
import static org.springframework.restdocs.payload.PayloadDocumentation.responseFields;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.mock.web.MockHttpSession;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.web.servlet.RequestBuilder;

import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.web.ControllerIntegrationTest;
import lcsb.mapviewer.web.api.NewApiDocs;

@RunWith(SpringJUnit4ClassRunner.class)
@ActiveProfiles("webCtdTestProfile")
public class NewChemicalControllerTest extends ControllerIntegrationTest {

  private static String TEST_PROJECT = "TEST_PROJECT";

  private Project project;

  @Before
  public void setup() {
    assumeTrue("DAPI credentials are not provided", isDapiConfigurationAvailable());
    project = createAndPersistProject(TEST_PROJECT);
  }

  @After
  public void tearDown() throws Exception {
    if (project != null) {
      removeProject(project);
    }
  }

  @Test
  public void testSearchChemicalsInProjectUrl() throws Exception {
    MockHttpSession session = createSession(BUILT_IN_TEST_ADMIN_LOGIN, BUILT_IN_TEST_ADMIN_PASSWORD);

    RequestBuilder request = get("/minerva/new_api/projects/{projectId}/chemicals:search?query=stilbene oxide", TEST_PROJECT)
        .session(session);

    mockMvc.perform(request)
        .andDo(document("new_api/projects/project_chemicals/search_by_query",
            getProjectPathParameters(),
            NewApiDocs.getChemicalFilter(),
            responseFields(NewApiDocs.getChemicalListResponse(""))))
        .andExpect(status().is2xxSuccessful())
        .andReturn().getResponse().getContentAsString();
  }
}
